import logging

import psycopg2
import uuid
import os

from psc_orgkey import encode_orgkey

ENV = os.getenv('ENV')

# Nothing in this file was done efficiently. But it's minimally needed and relatively flexible.


def get_device(org_key):
    # Use ENV vars
    conn = psycopg2.connect('')

    logging.debug("\nOpened database successfully")

    cur = conn.cursor()
    device_guid = uuid.uuid4().hex

    logging.debug("Trying to fetch existing device...")

    cur.execute('SELECT device_id, device_guid FROM device LIMIT 1')
    rows = cur.fetchall()

    if not rows:
        logging.debug("Device does not exist, so creating ...")

        cur.execute(f'''INSERT INTO device(device_type_id, device_owner_id, device_guid, name, email, email_type_id, session_id, create_time, kill_switch_on, org_key, deployment_type, properties)
                           VALUES (1, null, '{device_guid}', 'Device 1', 'device@scargoinc.com', null, null, NOW(), false, '{org_key}', 0, null);''')

        logging.debug(f'\nInserted device {device_guid} successfully')

        conn.commit()
        conn.close()

        return get_device(org_key)
    else:
        logging.debug("Device exists, returning device")
        device_id = 0
        device_guid = ''
        for row in rows:
            logging.debug(f'\ndevice_id {row[0]}')
            device_id = row[0]
            logging.debug(f'\ndevice_guid {row[1]}')
            device_guid = row[1]

        conn.commit()
        conn.close()

        return {
            "device_id": device_id,
            "device_guid": device_guid
        }


def get_org():
    # Use ENV vars
    conn = psycopg2.connect('')

    logging.debug("\nOpened database successfully")

    cur = conn.cursor()

    logging.debug("Trying to fetch existing org")

    cur.execute('SELECT org_id, org_key FROM organization LIMIT 1')
    rows = cur.fetchall()

    if not rows:
        cur.execute(f'''INSERT INTO organization(org_type_id, name, admin_contact_id, billing_contact_id, registration_password, analytics_profile, analytics_version, storage_profile, create_time, mode, create_admin, kill_switch_on, ransomware_config, uninstall_code, license_endpoints, effective_th_endpoints, license_expires)
                        values (1,'scargoinc.com',null,null,'AABBCC00','DOME','V2',null,NOW(),1,null,false,null,null,1,1,null)''')

        conn.commit()

        logging.debug(f'\nInserted org successfully')

        org = get_org()
        org_key = encode_orgkey(ENV, org['org_id'])

        cur.execute(f'''UPDATE organization SET org_key = '{org_key}' WHERE org_id = {org['org_id']} ''')
        org['org_key'] = org_key

        conn.commit()
        conn.close()

        return org
    else:
        logging.debug("Org exists, returning org")
        org_id = 0
        org_key = ''
        for row in rows:
            logging.debug(f'\norg_id {row[0]}')
            org_id = row[0]
            logging.debug(f'\norg_key {row[1]}')
            org_key = row[1]

        conn.commit()
        conn.close()

    return {
        "org_id": org_id,
        "org_key": org_key
    }


def get_policy(device_id, org_id):
    # Use ENV vars
    conn = psycopg2.connect('')

    logging.debug("\nOpened database successfully")

    cur = conn.cursor()

    logging.debug("Trying to fetch existing policy")

    cur.execute('SELECT device_group_id FROM device_group LIMIT 1')
    rows = cur.fetchall()

    if not rows:
        cur.execute(f'''INSERT INTO device_group(org_id, policy_control, is_default, priority, name, priority_level)
                        VALUES ({org_id}, true, true, 100, 'default', 1)''')

        conn.commit()

        logging.debug(f'\nInserted policy successfully')

        policy = get_policy(device_id, org_id)

        cur.execute(f''' INSERT INTO device_group_device(device_id, device_group_id) VALUES({device_id}, {policy['device_group_id']}) ''')

        conn.commit()
        conn.close()

        return policy
    else:
        logging.debug("policy exists, returning policy")
        policy_id = 0
        for row in rows:
            logging.debug(f'\npolicy_id {row[0]}')
            policy_id = row[0]

        conn.commit()
        conn.close()

    return {
        "device_group_id": policy_id
    }


def clean():
    conn = psycopg2.connect('')

    logging.debug("\nOpened database successfully")

    cur = conn.cursor()

    logging.debug("truncate organization")

    cur.execute('truncate organization cascade')
    conn.commit()
    cur.execute('truncate device cascade')
    conn.commit()
    cur.execute('truncate device_group cascade')
    conn.commit()

    conn.close()
