import logging
import os
import sys

import pytest
import requests

from test_data import get_device, get_org, get_policy, clean
from events import Events

from tenacity import retry, stop_after_attempt, wait_fixed

logging.getLogger().addHandler(logging.StreamHandler(sys.stdout))
logging.getLogger().setLevel(logging.DEBUG)

# If things get funky with the poke test, you can either cascade delete the organization, device, and device_guid
# tables manually, or uncomment this
# @pytest.fixture(scope='session', autouse=True)
# def clean_db():
#     clean()

@pytest.fixture
def registered_device():
    org = get_org()
    return get_device(org['org_key'])


@pytest.fixture
def registered_org():
    return get_org()


@pytest.fixture
def policy():
    org = get_org()
    device = get_device(org['org_key'])
    return get_policy(device['device_id'], org['org_id'])


@retry(stop=stop_after_attempt(2), wait=wait_fixed(1))
def test_events_external_hostname_with_ems_device_auth(registered_device, registered_org, policy):
    events = Events()
    events.generate_zip_for_event_creation()
    files = {"file": open(events.zip_file_location, "rb")}

    svc_events_url = f'https://{os.getenv("EXTERNAL_INGRESS_HOSTNAME")}/services/psc/v1/threathunter/events/'
    logging.debug(f"URL = {svc_events_url}")
    headers = {
        "Content-Type": "application/zip",
        "Device-Guid": registered_device['device_guid'],
        "Registration-ID": f'''{registered_org['org_id']}-{registered_device['device_id']}'''
    }
    response = requests.post(svc_events_url, headers=headers, files=files)

    os.remove(events.zip_file_location)

    assert response.status_code is 204, f'Expected events call to succeed.'
